--
-- © Copyright Peregrine I.T. Pty Ltd 2004-2018 except where explicitly stated otherwise.
-- Fantasy Grounds is Copyright © 2004-2018 SmiteWorks USA LLC.
-- Copyright to other material within this file may be held by other Individuals and/or Entities.
-- Nothing in or from this LUA file in printed, electronic and/or any other form may be used, copied,
--	transmitted or otherwise manipulated in ANY way without the explicit written consent of
--	Peregrine I.T. Pty Ltd or, where applicable, any and all other Copyright holders.
-- Please see the accompanying License for full details.
-- All rights reserved.
--
-- Called By: wcWeatherMain in xWeatherWindowClasses.xml
--

function onInit()
	fpUpdate();
end

function fpGetParentNodeDetails(sNodeName,sWindowClass)
	local sParentLink = "";
	local sParentWindowClass = "";
	local sParentNodeName = "";
	if sWindowClass == "wcLocation" then
		sParentLink = "lLocationParentLink";
	elseif sWindowClass == "wcTown" then
		sParentLink = "lTownParentLink";
	elseif sWindowClass == "wcBuilding" then
		sParentLink = "lBuildingParentLink";
	elseif sWindowClass == "wcPortal" then
		sParentLink = "lPortalParentLink";
	end
	if sParentLink == "" then
		return nil,nil;
	end
	sParentWindowClass,sParentNodeName = DB.getChild(sNodeName,sParentLink).getValue();
	return sParentWindowClass,DB.findNode(sParentNodeName);
end

function fpSetLocation(oDragData)
	local oThisNode = getDatabaseNode();
	local nDraggedPortalType = 1;
	local sParentWindowClass,sParentNodeName = oDragData.getShortcutData();
	local oParentNode = oDragData.getDatabaseNode();
	if sParentWindowClass == "wcPortal" then
		nDraggedPortalType = DB.getChild(sParentNodeName,"bPortalType1").getValue();
	end
	if oDragData.isType("shortcut") and
			(sParentWindowClass == "wcLocation" or
			sParentWindowClass == "wcTown" or
			sParentWindowClass == "wcPortal" and
			nDraggedPortalType == 1) and
			not MiscFunctions.fpGetReadOnlyState(oThisNode) then
		MiscFunctions.fpSetNodeParentDetails(oThisNode,true,"WeatherLocationParent",oDragData.getDescription(),sParentWindowClass,sParentNodeName);
		local aData = {};
		local sCalendar = Interface.getString("sNoneSetStr");
		local nDaysInYear = 365;
		local sYearMeasure = Interface.getString("sTimeDaysStr");
		local nHoursInDay = 24;
		local sDayMeasure = Interface.getString("sTimeHoursStr");
		local sSumSolDate = Interface.getString("sNoneSetStr");
		local nSumSolDate = 0;
		local bIsPlane = 0;
		local nTilt = 22.5;
		local sSpinDirection = Interface.getString("sSpinWestToEastStr");
		local sTemperature = Interface.getString("sTemperatureNormalStr");
		local sWetness = Interface.getString("sWetnessNormalStr");
		local sClimate = Interface.getString("sClimateTemperateStr");
		local nLatitude = 40;
		local sLatHemisphere = Interface.getString("sDirectionNorthStr");
		local sTerrain = Interface.getString("sTerrainPlainsStr");
		local snAltitudeExp = "2";
		local snAltitudeNum = "5";
		local sAltitudeMeasure = Interface.getString("sLengthFeetStr");
		while not (sParentWindowClass == nil or
				sParentWindowClass == "wcWorld" or
				sParentWindowClass == "wcPlane" or
				(sParentWindowClass == "wcLocation" or
				sParentWindowClass == "wcPortal") and
				oParentNode.getChild("bUseParentClimate") and
				oParentNode.getChild("bUseParentClimate").getValue() == 0) do
			sParentWindowClass,oParentNode = fpGetParentNodeDetails(sParentNodeName,sParentWindowClass);
		end
		if sParentWindowClass == "wcLocation" or
				sParentWindowClass == "wcPortal" then
			local 
			nLatitude = oParentNode.getChild("nLatitude").getValue();
			snAltitudeExp = oParentNode.getChild("snAltitudeExp").getValue();
			snAltitudeNum = oParentNode.getChild("snAltitudeNum").getValue();
			if oParentNode.getChild("sClimate") then
				sClimate = oParentNode.getChild("sClimate").getValue();
			end
			if oParentNode.getChild("sLatHemisphere") then
				sLatHemisphere = oParentNode.getChild("sLatHemisphere").getValue();
			end
			if oParentNode.getChild("sTerrain") then
				sTerrain = oParentNode.getChild("sTerrain").getValue();
			end
			if oParentNode.getChild("sAltitudeMeasure") then
				sAltitudeMeasure = oParentNode.getChild("sAltitudeMeasure").getValue();
			end
		end
		while not (sParentWindowClass == nil or
				sParentWindowClass == "wcWorld" or
				sParentWindowClass == "wcPlane") do
			sParentWindowClass,oParentNode = fpGetParentNodeDetails(oParentNode.getNodeName(),sParentWindowClass);
		end
		if sParentWindowClass == "wcWorld" then
			nDaysInYear = oParentNode.getChild("nDaysInYear").getValue();
			nHoursInDay = oParentNode.getChild("nHoursInDay").getValue();
			nTilt = oParentNode.getChild("nTilt").getValue();
			if oParentNode.getChild("sCalendar") then
				sCalendar = oParentNode.getChild("sCalendar").getValue();
			end
			if oParentNode.getChild("sYearMeasure") then
				sYearMeasure = oParentNode.getChild("sYearMeasure").getValue();
			end
			if oParentNode.getChild("sDayMeasure") then
				sDayMeasure = oParentNode.getChild("sDayMeasure").getValue();
			end
			if oParentNode.getChild("sSumSolDate") then
				sSumSolDate = oParentNode.getChild("sSumSolDate").getValue();
			end
			if oParentNode.getChild("nSumSolDate") then
				nSumSolDate = oParentNode.getChild("nSumSolDate").getValue();
			end
			if oParentNode.getChild("sSpinDirection") then
				sSpinDirection = oParentNode.getChild("sSpinDirection").getValue();
			end
			if oParentNode.getChild("sTemperature") then
				sTemperature = oParentNode.getChild("sTemperature").getValue();
			end
			if oParentNode.getChild("sWetness") then
				sWetness = oParentNode.getChild("sWetness").getValue();
			end
		elseif sParentWindowClass == "wcPlane" then
			bIsPlane = 1;
			if oParentNode.getChild("sSpinDirection") then
				sSpinDirection = oParentNode.getChild("sSpinDirection").getValue();
			end
			if oParentNode.getChild("sTemperature") then
				sTemperature = oParentNode.getChild("sTemperature").getValue();
			end
			if oParentNode.getChild("sWetness") then
				sWetness = oParentNode.getChild("sWetness").getValue();
			end
		end
		DB.setValue(oThisNode,"sCalendar","string",sCalendar);
		DB.setValue(oThisNode,"nDaysInYear","number",nDaysInYear);
		DB.setValue(oThisNode,"sYearMeasure","string",sYearMeasure);
		DB.setValue(oThisNode,"nHoursInDay","number",nHoursInDay);
		DB.setValue(oThisNode,"sDayMeasure","string",sDayMeasure);
		DB.setValue(oThisNode,"sSumSolDate","string",sSumSolDate);
		DB.setValue(oThisNode,"nSumSolDate","string",nSumSolDate);
		DB.setValue(oThisNode,"bIsPlane","number",bIsPlane);
		DB.setValue(oThisNode,"nTilt","number",nTilt);
		DB.setValue(oThisNode,"sSpinDirection","string",sSpinDirection);
		DB.setValue(oThisNode,"sTemperature","string",sTemperature);
		DB.setValue(oThisNode,"sWetness","string",sWetness);
		DB.setValue(oThisNode,"sClimate","string",sClimate);
		DB.setValue(oThisNode,"nLatitude","number",nLatitude);
		DB.setValue(oThisNode,"sLatHemisphere","string",sLatHemisphere);
		DB.setValue(oThisNode,"sTerrain","string",sTerrain);
		DB.setValue(oThisNode,"snAltitudeExp","number",snAltitudeExp);
		DB.setValue(oThisNode,"snAltitudeNum","number",snAltitudeNum);
		DB.setValue(oThisNode,"sAltitudeMeasure","string",sAltitudeMeasure);
	end
	return true;
end

function fpShowEffectiveTemp()
	local bShowEffectiveTemp = OptionsManager.isOption("HUMIDITYON","on") and
			(nEffectiveTempLow.getValue() > nTempLow.getValue() or
			nEffectiveTempHigh.getValue() > nTempHigh.getValue()) or
			OptionsManager.isOption("WINDCHILLON","on") and
			(nEffectiveTempLow.getValue() < nTempLow.getValue() or
			nEffectiveTempHigh.getValue() < nTempHigh.getValue());
	nEffectiveTempLow.setVisible(bShowEffectiveTemp);
	nEffectiveTempHigh.setVisible(bShowEffectiveTemp);
	sEffectiveTemperatureLabel.setVisible(bShowEffectiveTemp);
	sEffectiveToStr.setVisible(bShowEffectiveTemp);
	sEffectiveTempMeasure.setVisible(bShowEffectiveTemp);
end

function fpShowWind(bShowWind)
	sWindDirection.setComboBoxVisible(bShowWind);
	sWindDirectionLabel.setVisible(bShowWind);
	nWindGustingSpeed.setVisible(bShowWind);
	nWindGustingSpeedLabel.setVisible(bShowWind);
	sWindGustingSpeedMeasure.setVisible(bShowWind);
end

function fpUpdate()
	if sSpecialWeather then
		local bReadOnly = MiscFunctions.fpGetReadOnlyState(getDatabaseNode());
		local bSection1 = false;
		local bSection2 = false;
		local bShowCal = (bUseCal.getValue() == 1);
--		local bShowCal = (bUseCal.getValue() == 1 or bUseCal.getValue() == true);
		local bLocationsLoaded = MiscFunctions.fpIsExtensionLoaded("Locations",true);
		sCurrentDate.setFrame("fieldlight",7,5,7,5);
		sSunrise.setFrame("fieldlight",7,5,7,5);
		sSunset.setFrame("fieldlight",7,5,7,5);
		sTempMeasure.setFrame("fieldlight",7,5,7,5);
		nEffectiveTempLow.setFrame("fieldlight",7,5,7,5);
		nEffectiveTempHigh.setFrame("fieldlight",7,5,7,5);
		sEffectiveTempMeasure.setFrame("fieldlight",7,5,7,5);
		sWindSpeedMeasure.setFrame("fieldlight",7,5,7,5);
		nWindGustingSpeed.setFrame("fieldlight",7,5,7,5);
		sWindGustingSpeedMeasure.setFrame("fieldlight",7,5,7,5);
		sRainType.setFrame("fieldlight",7,5,7,5);
		sRainHow.setFrame("fieldlight",7,5,7,5);
		sSpecialWeather.setFrame("fieldlight",7,5,7,5);
		if bReadOnly then
			sCurrentDate.setFrame(nil);
			sSunrise.setFrame(nil);
			sSunset.setFrame(nil);
			sTempMeasure.setFrame(nil);
			nEffectiveTempLow.setFrame(nil);
			nEffectiveTempHigh.setFrame(nil);
			sEffectiveTempMeasure.setFrame(nil);
			sWindSpeedMeasure.setFrame(nil);
			nWindGustingSpeed.setFrame(nil);
			sWindGustingSpeedMeasure.setFrame(nil);
			sRainType.setFrame(nil);
			sRainHow.setFrame(nil);
			sSpecialWeather.setFrame(nil);
		end
		if MiscFunctions.fpUpdateCtrl(self,"sWeatherLocationName",bReadOnly,bLocationsLoaded) then
			bSection1 = true;
		end
		if MiscFunctions.fpUpdateCtrl(self,"sWeatherLocationParentName",bReadOnly,not bLocationsLoaded) then
			bSection1 = true;
		end
		sWeatherLocationNameLabel.setVisible(bSection1);
		lWeatherLocationParentLink.setVisible(sWeatherLocationParentName.isVisible());
		bcDelLinkWeatherLocationParent.setVisible(not bReadOnly and bLocationsLoaded);
		sCurrentDate.setAnchor("right","gAnchorRight","left","",-95);
		if bReadOnly then
			sCurrentDate.setAnchor("right","gAnchorRight","left","",-12);
		end
		sHumidity.setComboBoxReadOnly(bReadOnly);
		sWindDirection.setComboBoxReadOnly(bReadOnly);
		sRainAmount.setComboBoxReadOnly(bReadOnly);
		bUseCal.setVisible(not bReadOnly);
		bUseCalLabel.setVisible(not bReadOnly);
		sCurrentDate.setVisible(bShowCal and (sCurrentDate.getValue() ~= Interface.getString("sNoneSetStr") or not bReadOnly));
		if sCurrentDate.isVisible() then
			bSection2 = true;
		end
		sCurrentDateLabel.setVisible(sCurrentDate.isVisible());
		btButCalGetCurrentDate.setVisible(bShowCal and not bReadOnly)
		nWeatherDay.setVisible(not bShowCal and (nWeatherDay.getValue() ~= 0 or not bReadOnly));
		nWeatherDay.setReadOnly(bReadOnly);
		if nWeatherDay.isVisible() then
			bSection2 = true;
		end
		nWeatherDayLabel.setVisible(nWeatherDay.isVisible());
		nWeatherYear.setVisible(not bShowCal and (nWeatherYear.getValue() ~= 0 or not bReadOnly));
		nWeatherYear.setReadOnly(bReadOnly);
		if nWeatherYear.isVisible() then
			bSection2 = true;
		end
		nWeatherYearLabel.setVisible(nWeatherYear.isVisible());
		nTempLow.setReadOnly(bReadOnly);
		nTempHigh.setReadOnly(bReadOnly);
		fpShowEffectiveTemp();
		nWindSpeed.setReadOnly(bReadOnly);
		nWindSpeed.setVisible(true);
		nWindSpeedLabel.setVisible(true);
		sRainAmount.onValueChanged()
		divider1.setVisible(bSection1);
		divider2.setVisible(bSection2);
		divider7.setVisible(sSpecialWeather.isVisible());
		fpShowWind((nWindSpeed.getValue() ~= 0));
	end
end
