--
-- © Copyright Peregrine I.T. Pty Ltd 2004-2018 except where explicitly stated otherwise.
-- Fantasy Grounds is Copyright © 2004-2018 SmiteWorks USA LLC.
-- Copyright to other material within this file may be held by other Individuals and/or Entities.
-- Nothing in or from this LUA file in printed, electronic and/or any other form may be used, copied,
--	transmitted or otherwise manipulated in ANY way without the explicit written consent of
--	Peregrine I.T. Pty Ltd or, where applicable, any and all other Copyright holders.
-- Please see the accompanying License for full details.
-- All rights reserved.
--
-- Called By: xWeatherFiles.xml
--

local DBVERSION = 2;
local EXTENSION = "Weather";
local DEFAULTMEASUREMENTUNIT = "Foot";
local SUMSOLDAY = 183;

aExport = {};
sBtnArrowRight = "btnArrowRightCoreRPG";
sBtnArrowRightDown = "btnArrowRightDownCoreRPG";
sBtnWeather = "btnWeatherCoreRPG";
sBtnWeatherDown = "btnWeatherDownCoreRPG";
sITabDetails = "iTabDetailsCoreRPG";
sITabSound = "iTabSoundCoreRPG";

aSubRecords = {
	["weather"] = {
		aDataMap = {"MJB_Weather"},
		sRecordDisplayClass = "wcWeather",
		aGMListButtons = {"tButLicenseLibrary"},
		aPlayerListButtons = {"tButLicenseLibrary"}
	}
}
local aCopyMappings = {};

function onInit()
	if User.isHost() then
		local oNode = DB.findNode("MJB_Extensions");
		if DB.findNode("weather") then
			fpDoPatch1();
		elseif oNode == nil or
				DB.getChild(oNode,EXTENSION) == nil then
			DB.setValue(DB.createNode("MJB_Extensions"),EXTENSION,"number",DBVERSION);
			DB.createNode("MJB_Weather");
		elseif DB.getValue(oNode,EXTENSION) < DBVERSION then
			fpDoPatch2();
		end
		if ExportManager.registerExportNode then
			ExportManager.registerExportNode({name = "MJB_Weather",class = "wcWeather",label = "Weather"});
		end
	end
	fpSetRulesetGraphics();
	fpMain();
end

function fpSetRulesetGraphics()
	local sRuleset = Interface.getRuleset();
	if MiscFunctions.fpIsExtensionLoaded("Theme_Simple_Brown") then
		sRuleset = "SimpBrown";
	elseif MiscFunctions.fpIsExtensionLoaded("Theme_Simple_Gray") then
		sRuleset = "SimpGrey";
	elseif sRuleset == "5E" then
		if MiscFunctions.fpIsExtensionLoaded("5E_Theme_Wizards") then
			sRuleset = "5EWotC"
		else
			sRuleset = "5E";
		end
	elseif sRuleset == "Castles and Crusades" then
		sRuleset = "CnC";
	elseif sRuleset == "CallOfCthulhu" then
		sRuleset = "CoC";
	elseif sRuleset == "CallOfCthulhu7E" then
		sRuleset = "CoC";
	elseif sRuleset == "SavageWorlds" then
		sRuleset = "SW";
--[[ Depreciated - Remove in v3.2.0+ --]]
elseif sRuleset == "WH40KDarkHeresy" then
	sRuleset = "40KDH";
elseif sRuleset == "WHFRP 2ed" then
	sRuleset = "WFRP";
--[[ To Here --]]
	else
		sRuleset = "CoreRPG";
	end
	sBtnArrowRight = "btnArrowRight" .. sRuleset;
	sBtnArrowRightDown = "btnArrowRightDown" .. sRuleset;
	sBtnWeather = "btnWeather" .. sRuleset;
	sBtnWeatherDown = "btnWeatherDown" .. sRuleset;
	sITabDetails = "iTabDetails" .. sRuleset;
	sITabSound = "iTabSound" .. sRuleset;
	return;
end

-- Patch Functions --

function fpConvertToVersion2()
	aCopyMappings = {};
	fpCopyOldNodes("weather","MJB_Weather");
	fpUpdateWeather();
end

function fpCopyOldNodes(sSourceParent,sDestinationParent)
	local oSourceNode = DB.findNode(sSourceParent);
	local oNewNode = nil;
	if oSourceNode then
		for kKey,vValue in pairs(oSourceNode.getChildren()) do
			oNewNode = DB.createChild(sDestinationParent);
			DB.copyNode(vValue,oNewNode);
			aCopyMappings[vValue.getNodeName()] = oNewNode.getNodeName();
		end
	oSourceNode.delete();
	end
end

function fpDoPatch1()
	DB.backup();
	fpConvertToVersion2();
	DB.setValue(DB.createNode("MJB_Extensions"),EXTENSION,"number",DBVERSION);
	return;
end

function fpDoPatch2()
	DB.backup();
	DB.setValue(DB.createNode("MJB_Extensions"),EXTENSION,"number",DBVERSION);
	return;
end

function fpConvertToSciNum(oNode,sSourceName)
	local nNum,nExp = MiscFunctions.fpSimplifySciNotation(oNode.getChild(sSourceName).getValue(),0);
	oNode.createChild("s" .. sSourceName .. "Num","number").setValue(nNum);
	oNode.createChild("s" .. sSourceName .. "Exp","number").setValue(nExp);
	oNode.getChild(sSourceName).delete();
	return;
end

function fpIsOldCopy(sNodeName)
	for kKey,vValue in pairs(aCopyMappings) do
		if sNodeName == vValue then
			return true;
		end
	end
	return false;
end

function fpRenameNode(oParentNode,sSourceName,sDestinationName)
	local oSourceNode = oParentNode.getChild(sSourceName);
	DB.copyNode(oSourceNode,oParentNode.getNodeName() .. "." .. sDestinationName);
	oSourceNode.delete();
	return;
end

function fpUpdateLinkage(oNode)
	local sLinkedClass,sLinkedNodeName = oNode.getValue();
	if sLinkedClass ~= "" and
			sLinkedNodeName ~= "" then
		oNode.setValue(sLinkedClass,aCopyMappings[sLinkedNodeName]);
	end
	return;
end

function fpUpdateWeather()
	local aClimateMappings = {};
	aClimateMappings["temperate"] = Interface.getString("sClimateTemperateStr");
	aClimateMappings["subtropical"] = Interface.getString("sClimateSubtropicalStr");
	aClimateMappings["tropical"] = Interface.getString("sClimateTropicalStr");
	aClimateMappings["arctic"] = Interface.getString("sClimateArcticStr");
	aClimateMappings["subarctic"] = Interface.getString("sClimateSubarcticStr");
	local aTimeMappings = {};
	aTimeMappings["days"] = Interface.getString("sTimeDaysStr");
	aTimeMappings["minutes"] = Interface.getString("sTimeMinutesStr");
	aTimeMappings["hours"] = Interface.getString("sTimeHoursStr");
	local aWetnessMappings = {};
	aWetnessMappings["w"] = Interface.getString("sWetnessDampStr");
	aWetnessMappings["s"] = Interface.getString("sWetnessSoppingStr");
	aWetnessMappings["p"] = Interface.getString("sWetnessParchedStr");
	aWetnessMappings["d"] = Interface.getString("sWetnessDryStr");
	local aSpinMappings = {};
	aSpinMappings["easttowest"] = Interface.getString("sSpinEastToWestStr");
	aSpinMappings["none"] = Interface.getString("sSpinNoSpinStr");
	local aTemperatureMappings = {};
	aTemperatureMappings["warm"] = Interface.getString("sTemperatureHotStr");
	aTemperatureMappings["hot"] = Interface.getString("sTemperatureHotStr");
	aTemperatureMappings["boiling"] = Interface.getString("sTemperatureBoilingStr");
	aTemperatureMappings["cool"] = Interface.getString("sTemperatureFreezingStr");
	aTemperatureMappings["cold"] = Interface.getString("sTemperatureColdStr");
	aTemperatureMappings["freezing"] = Interface.getString("sTemperatureCoolStr");
	local aTerrainMappings = {};
	aTerrainMappings["coast"] = Interface.getString("sTerrainCoastStr");
	aTerrainMappings["desert"] = Interface.getString("sTerrainDesertStr");
	aTerrainMappings["forest"] = Interface.getString("sTerrainForestStr");
	aTerrainMappings["hills"] = Interface.getString("sTerrainHillsStr");
	aTerrainMappings["cmarsh"] = Interface.getString("sTerrainCoastMarshStr");
	aTerrainMappings["fmarsh"] = Interface.getString("sTerrainForestMarshStr");
	aTerrainMappings["pmarsh"] = Interface.getString("sTerrainPlainsMarshStr");
	aTerrainMappings["mountains"] = Interface.getString("sTerrainMountainsStr");
	local aRainMappings = {};
	aRainMappings["t"] = Interface.getString("sPrecipitationAmountTraceStr");
	aRainMappings["l"] = Interface.getString("sPrecipitationAmountLightStr");
	aRainMappings["m"] = Interface.getString("sPrecipitationAmountModerateStr");
	aRainMappings["h"] = Interface.getString("sPrecipitationAmountHeavyStr");
	aRainMappings["d"] = Interface.getString("sPrecipitationAmountDownpourStr");
	local aHumidityMappings = {};
	aHumidityMappings["h"] = Interface.getString("sHumidityHeavyStr");
	aHumidityMappings["s"] = Interface.getString("sHumiditySaturatedStr");
	aHumidityMappings["d"] = Interface.getString("sHumidityDryStr");
	aHumidityMappings["l"] = Interface.getString("sHumidityLowStr");
	local aWindMappings = {};
	aWindMappings["ne"] = Interface.getString("sDirectionNorthEastStr");
	aWindMappings["e"] = Interface.getString("sDirectionEastStr");
	aWindMappings["se"] = Interface.getString("sDirectionSouthEastStr");
	aWindMappings["s"] = Interface.getString("sDirectionSouthStr");
	aWindMappings["sw"] = Interface.getString("sDirectionSouthWestStr");
	aWindMappings["w"] = Interface.getString("sDirectionWestStr");
	aWindMappings["nw"] = Interface.getString("sDirectionNorthWestStr");
	for kKey,vValue in pairs(DB.getChildren("MJB_Weather")) do
		if fpIsOldCopy(vValue.getNodeName()) then
			fpRenameNode(vValue,"nSummerSolDay","nSumSolDay");
			fpRenameNode(vValue,"sSummerSolDate","sSumSolDate");
			fpRenameNode(vValue,"slWeatherLocationParentLink","lWeatherLocationParentLink");
			fpConvertToSciNum(vValue,"nAltitude");
			fpUpdateLinkage(vValue.getChild("lWeatherLocationParentLink"));
			if vValue.getChild("sLatHemisphere") then
				vValue.getChild("sLatHemisphere").setValue(Interface.getString("sDirectionSouthStr"));
			else
				vValue.createChild("sLatHemisphere","string").setValue(Interface.getString("sDirectionNorthStr"));
			end
			if vValue.getChild("sAltitudeMeasure") then
				vValue.getChild("sAltitudeMeasure").setValue(Interface.getString("sLengthMetreStr"));
			else
				vValue.createChild("sAltitudeMeasure","string").setValue(Interface.getString("sLengthFeetStr"));
			end
			oNode = vValue.getChild("sClimate");
			if oNode then
				oNode.setValue(aClimateMappings[oNode.getValue()]);
			else
				vValue.createChild("sClimate","string").setValue(Interface.getString("sClimateTemperateStr"));
			end
			oNode = vValue.getChild("sDayMeasure");
			if oNode then
				oNode.setValue(aTimeMappings[oNode.getValue()]);
			else
				vValue.createChild("sDayMeasure","string").setValue(Interface.getString("sTimeHoursStr"));
			end
			oNode = vValue.getChild("sWetness");
			if oNode then
				oNode.setValue(aWetnessMappings[oNode.getValue()]);
			else
				vValue.createChild("sWetness","string").setValue(Interface.getString("sNormalStr"));
			end
			oNode = vValue.getChild("sSpinDirection");
			if oNode then
				oNode.setValue(aSpinMappings[oNode.getValue()]);
			else
				vValue.createChild("sSpinDirection","string").setValue(Interface.getString("sSpinWestToEastStr"));
			end
			oNode = vValue.getChild("sTemperature");
			if oNode then
				oNode.setValue(aTemperatureMappings[oNode.getValue()]);
			else
				vValue.createChild("sTemperature","string").setValue(Interface.getString("sNormalStr"));
			end
			oNode = vValue.getChild("sTerrain");
			if oNode then
				oNode.setValue(aTerrainMappings[oNode.getValue()]);
			else
				vValue.createChild("sTerrain","string").setValue(Interface.getString("sTerrainPlainsStr"));
			end
			oNode = vValue.getChild("sYearMeasure");
			if oNode then
				oNode.setValue(aTimeMappings[oNode.getValue()]);
			else
				vValue.createChild("sYearMeasure","string").setValue(Interface.getString("sTimeDaysStr"));
			end
			oNode = vValue.getChild("sRainAmount");
			if oNode then
				oNode.setValue(aRainMappings[oNode.getValue()]);
			else
				vValue.createChild("sRainAmount","string").setValue(Interface.getString("sNoneStr"));
			end
			oNode = vValue.getChild("sHumidity");
			if oNode then
				oNode.setValue(aHumidityMappings[oNode.getValue()]);
			else
				vValue.createChild("sHumidity","string").setValue(Interface.getString("sHumidityMediumStr"));
			end
			oNode = vValue.getChild("sWindDirection");
			if oNode then
				oNode.setValue(aWindMappings[oNode.getValue()]);
			else
				vValue.createChild("sWindDirection","string").setValue(Interface.getString("sDirectionNorthStr"));
			end
		end
	end
	return;
end

-- End Of Patch Functions --

function fpMain()
	for kKey,vValue in pairs(aSubRecords) do
		LibManager.fpSetRecordTypeInfo(kKey,vValue);
	end
	if not User.isLocal() then
		DesktopManager.registerStackShortcut2(sBtnWeather,sBtnWeatherDown,"sSidebarTooltipWeatherStr","wcLibMasterList","MJB_Weather");
	end
	fpRegisterWeatherOptions();
	GameSystem.actions["WeatherRollToday"] = {};
	GameSystem.actions["WeatherRollTomorrow"] = {};
	return;
end

function fpAdjustedDayOfYear(nDay,nYearLength,nSumSol)
	return math.ceil((nDay+1.5*nYearLength-nSumSol)%nYearLength);
end

function fpAdjustRainfallForWetness(oNode,sRainFall)
	local sTmpRainfall = sRainFall;
	local sWetness = Interface.getString("sNormalStr");
	if oNode.getChild("sWetness") then
		sWetness = oNode.getChild("sWetness").getValue();
	end
	if sWetness == Interface.getString("sWetnessParchedStr") then
		sTmpRainfall = fpAdjustRainfallForWetnessParched(sRainFall);
	elseif sWetness == Interface.getString("sWetnessDryStr") then
		sTmpRainfall = fpAdjustRainfallForWetnessDry(sRainFall);
	elseif sWetness == Interface.getString("sWetnessDampStr") then
		sTmpRainfall = fpAdjustRainfallForWetnessDamp(sRainFall);
	elseif sWetness == Interface.getString("sWetnessSoppingStr") then
		sTmpRainfall = fpAdjustRainfallForWetnessSopping(sRainFall);
	end
	return sTmpRainfall;
end

function fpAdjustRainfallForWetnessDamp(sRainFall)
	local sTmpRainfall = Interface.getString("sPrecipitationAmountDownpourStr");
	if sRainFall == Interface.getString("sNoneStr") then
		sTmpRainfall = Interface.getString("sPrecipitationAmountTraceStr");
	elseif sRainFall == Interface.getString("sPrecipitationAmountTraceStr") then
		sTmpRainfall = Interface.getString("sPrecipitationAmountLightStr");
	elseif sRainFall == Interface.getString("sPrecipitationAmountLightStr") then
		sTmpRainfall = Interface.getString("sPrecipitationAmountModerateStr");
	elseif sRainFall == Interface.getString("sPrecipitationAmountModerateStr") then
		sTmpRainfall = Interface.getString("sPrecipitationAmountHeavyStr");
	end
	return sTmpRainfall;
end

function fpAdjustRainfallForWetnessDry(sRainFall)
	local sTmpRainfall = Interface.getString("sNoneStr");
	if sRainFall == Interface.getString("sPrecipitationAmountDownpourStr") then
		sTmpRainfall = Interface.getString("sPrecipitationAmountHeavyStr");
	elseif sRainFall == Interface.getString("sPrecipitationAmountHeavyStr") then
		sTmpRainfall = Interface.getString("sPrecipitationAmountModerateStr");
	elseif sRainFall == Interface.getString("sPrecipitationAmountModerateStr") then
		sTmpRainfall = Interface.getString("sPrecipitationAmountLightStr");
	elseif sRainFall == Interface.getString("sPrecipitationAmountLightStr") then
		sTmpRainfall = Interface.getString("sPrecipitationAmountTraceStr");
	end
	return sTmpRainfall;
end

function fpAdjustRainfallForWetnessParched(sRainFall)
	local sTmpRainfall = Interface.getString("sNoneStr");
	if sRainFall == Interface.getString("sPrecipitationAmountDownpourStr") then
		sTmpRainfall = Interface.getString("sPrecipitationAmountModerateStr");
	elseif sRainFall == Interface.getString("sPrecipitationAmountHeavyStr") then
		sTmpRainfall = Interface.getString("sPrecipitationAmountLightStr");
	elseif sRainFall == Interface.getString("sPrecipitationAmountModerateStr") then
		sTmpRainfall = Interface.getString("sPrecipitationAmountTraceStr");
	end
	return sTmpRainfall;
end

function fpAdjustRainfallForWetnessSopping(sRainFall)
	local sTmpRainfall = Interface.getString("sPrecipitationAmountDownpourStr");
	if sRainFall == Interface.getString("sNoneStr") then
		sTmpRainfall = Interface.getString("sPrecipitationAmountLightStr");
	elseif sRainFall == Interface.getString("sPrecipitationAmountTraceStr") then
		sTmpRainfall = Interface.getString("sPrecipitationAmountModerateStr");
	elseif sRainFall == Interface.getString("sPrecipitationAmountLightStr") then
		sTmpRainfall = Interface.getString("sPrecipitationAmountHeavyStr");
	end
	return sTmpRainfall;
end

function fpAdjustTempForAltitude(oNode,nLow,nHigh,nTempChange)
	local nAltitudeNum = oNode.getChild("snAltitudeNum").getValue();
	local nAltitudeExp = oNode.getChild("snAltitudeExp").getValue();
	local sAltitudeMeasure = oNode.getChild("sAltitudeMeasure").getValue()
	if MiscFunctions.fpIsExtensionLoaded("DOEWMC",true) then
		local sFootID = WMCManager.fpGetL2IDFromName(WMCManager.LENGTHSYSTEM,DEFAULTMEASUREMENTUNIT);
		if sAltitudeMeasure ~= nil and
				sFootID ~= nil then
			nAltitudeNum,nAltitudeExp = WMCManager.fpConvertUnits(WMCManager.LENGTHSYSTEM,sAltitudeMeasure,DEFAULTMEASUREMENTUNIT,nAltitudeNum,nAltitudeExp)
		end
	elseif sAltitudeMeasure == Interface.getString("sLengthMetreStr") then
		nAltitudeNum = nAltitudeNum*3.2808399;
	end
	local nAltitudeTempChange = nTempChange-math.floor(3*nAltitudeNum*10^(nAltitudeExp-3)+0.5);
	return nLow+nAltitudeTempChange,nHigh+nAltitudeTempChange;
end

function fpAdjustTempForHotness(oNode,nLow,nHigh)
	local sTempSetting = oNode.getChild("sTemperature").getValue();
	if sTempSetting == Interface.getString("sTemperatureFreezingStr") then
		nLow = nLow-100;
		nHigh = nHigh-100;
	elseif sTempSetting == Interface.getString("sTemperatureColdStr") then
		nLow = nLow-50;
		nHigh = nHigh-50;
	elseif sTempSetting == Interface.getString("sTemperatureCoolStr") then
		nLow = nLow-25;
		nHigh = nHigh-25;
	elseif sTempSetting == Interface.getString("sTemperatureHotStr") then
		nLow = nLow+50;
		nHigh = nHigh+50;
	elseif sTempSetting == Interface.getString("sTemperatureBoilingStr") then
		nLow = nLow+100;
		nHigh = nHigh+100;
	end
	return nLow,nHigh;
end

function fpCalcWeather(oNode)
	local nDie1 = oNode.getChild("nDie1").getValue();
	local nDie2 = oNode.getChild("nDie2").getValue();
	local nDie4 = oNode.getChild("nDie4").getValue();
	local nDie6 = oNode.getChild("nDie6").getValue();
	local nDieTotal = nDie1+nDie2;
	local oYNode = fpGetNode(oNode,true);
	local nYearLength = fpGetYearLength(oNode);
	local nAdjustedDay = fpAdjustedDayOfYear(oNode.getChild("nWeatherDay").getValue() or 1,nYearLength,oNode.getChild("nSumSolDay").getValue() or SUMSOLDAY);
	local nMonth = math.ceil(12*nAdjustedDay/nYearLength);
	local sSeason = fpGetSeason(nMonth);
	local sClimateZone = fpGetClimateZone(oNode,nAdjustedDay,nYearLength);
	local sTempTerrain,sRainTerrain = fpGetTerrain(oNode,sClimateZone);
	local sSpinDirection = fpGetSpinDirection(oNode);
	local bNorthHemisphere = true;
	if oNode.getChild("sLatHemisphere") then
		bNorthHemisphere = (oNode.getChild("sLatHemisphere").getValue() == Interface.getString("sDirectionNorthStr"));
	end
	local sSpecialWeatherCode,nSpecialWeatherDuration = fpSetSpecialWeatherCode(oNode,oYNode,nDieTotal,nDie6,sClimateZone,sRainTerrain,sSeason);
	local nWindSpeed = fpSetWindSpeeed(oNode,oYNode,nDieTotal,nDie6,sClimateZone,sSpecialWeatherCode,nSpecialWeatherDuration);
	fpSetTemp(oNode,oYNode,nDieTotal,nDie6,sClimateZone,nMonth,sTempTerrain,sSpecialWeatherCode,nSpecialWeatherDuration);
	fpSetWindDirection(oNode,nDieTotal,nDie4,sClimateZone,nMonth,sSpinDirection,bNorthHemisphere,sSpecialWeatherCode,nSpecialWeatherDuration);
	fpSetRainFall(oNode,oYNode,nDieTotal,nDie1,nDie2,nDie4%2,sClimateZone,sRainTerrain,sSeason,sSpecialWeatherCode);
	fpSetHumidity(oNode,sClimateZone,sRainTerrain,sSeason);
	fpSetSpecialWeatherText(oNode,sRainTerrain,nWindSpeed,sSpecialWeatherCode);
end

function fpChangeTempMeasure()
	local bCtoF = OptionsManager.isOption("TEMPMEASURE","fahrenheit");
	local sTempMeasure = Interface.getString("sOptionValueCelsius");
	if bCtoF then
		sTempMeasure = Interface.getString("sOptionValueFahrenheit");
	end
	for kKey,oNode in pairs(DB.getChildren("MJB_Weather")) do
		DB.setValue(oNode,"nEffectiveTempLow","number",fpConvertCAndF(bCtoF,DB.getChild(oNode,"nEffectiveTempLow").getValue()));
		DB.setValue(oNode,"nEffectiveTempHigh","number",fpConvertCAndF(bCtoF,DB.getChild(oNode,"nEffectiveTempHigh").getValue()));
		DB.setValue(oNode,"nTempLow","number",fpConvertCAndF(bCtoF,DB.getChild(oNode,"nTempLow").getValue()));
		DB.setValue(oNode,"nTempHigh","number",fpConvertCAndF(bCtoF,DB.getChild(oNode,"nTempHigh").getValue()));
		DB.setValue(oNode,"sTempMeasure","string",sTempMeasure);
	end
end

function fpChangeWindSpeedMeasure()
	local bKtoM = OptionsManager.isOption("WINDMEASURE","mph");
	local sWindSpeedMeasure = Interface.getString("sOptionValueKPH");
	if bKtoM then
		sWindSpeedMeasure = Interface.getString("sOptionValueMPH");
	end
	for kKey,oNode in pairs(DB.getChildren("MJB_Weather")) do
		DB.setValue(oNode,"nWindSpeed","number",fpConvertKPHAndMPH(bKtoM,DB.getChild(oNode,"nWindSpeed").getValue()));
		DB.setValue(oNode,"nWindGustingSpeed","number",fpConvertKPHAndMPH(bKtoM,DB.getChild(oNode,"nWindGustingSpeed").getValue()));
		DB.setValue(oNode,"sWindSpeedMeasure","string",sWindSpeedMeasure);
	end
end

function fpConvertCAndF(bCtoF,nValue)
	local nNewValue;
	if bCtoF then
		nNewValue = nValue*9/5+32;
	else
		nNewValue = (nValue-32)*5/9;
	end
	return math.floor(nNewValue+0.5);
end

function fpConvertKPHAndMPH(bKtoM,nValue)
	local nNewValue;
	if bKtoM then
		nNewValue = nValue*5/9;
	else
		nNewValue = nValue*9/5;
	end
	return math.floor(nNewValue+0.5);
end

function fpCreateTomorrowsWeather(oOldNode,aRoll)
	local oNewNode = DB.createChild("MJB_Weather");
	local nMonthDay = oOldNode.getChild("nMonthDay").getValue()+1;
	local nWeatherDay = oOldNode.getChild("nWeatherDay").getValue()+1;
	local nWeatherMonth = oOldNode.getChild("nWeatherMonth").getValue();
	local nWeatherYear = oOldNode.getChild("nWeatherYear").getValue();
	local sCalendar = oOldNode.getChild("sCalendar").getValue();
	DB.copyNode(oOldNode,oNewNode);
	oOldNode.createChild("hsTomorrowLink","string").setValue(oNewNode.getNodeName());
	oNewNode.getChild("hsYesterdayLink").setValue(oOldNode.getNodeName());
	if nWeatherDay > oOldNode.getChild("nDaysInYear").getValue() then
		nWeatherDay = 1;
		nWeatherMonth = 1;
		nMonthDay = 1;
		nWeatherYear = nWeatherYear+1;
	elseif nMonthDay > CalendarManager.getDaysInMonth(nWeatherMonth) then
		nMonthDay = 1;
		nWeatherMonth = nWeatherMonth+1;
	end
	if sCalendar ~= "" and
			sCalendar ~= Interface.getString("sNoneSetStr") then
		oNewNode.getChild("sCurrentDate").setValue(CalendarManager.getDateString(DB.getValue("calendar.current.epoch",""),nWeatherYear,nWeatherMonth,nMonthDay));
	end
	oNewNode.createChild("bLocked","number").setValue(1);
	oNewNode.createChild("hnDaysAtMax","number").setValue(0);
	oNewNode.createChild("hnDaysAtMin","number").setValue(0);
	oNewNode.createChild("hnDaysSinceCyclone","number").setValue(math.min(oOldNode.getChild("hnDaysSinceCyclone").getValue()+1,31));
	oNewNode.createChild("hnDaysSinceDroughtTraceRainfall","number").setValue(math.min(oOldNode.getChild("hnDaysSinceDroughtTraceRainfall").getValue()+1,6));
	oNewNode.createChild("hnSpecialWeatherDuration","number").setValue(math.max(oOldNode.getChild("hnSpecialWeatherDuration").getValue()-1,0));
	oNewNode.createChild("hsSpecialWeatherCode","string").setValue("n");
	oNewNode.createChild("nDie1","number").setValue(aRoll.aDice[1].result);
	oNewNode.createChild("nDie2","number").setValue(aRoll.aDice[2].result);
	oNewNode.createChild("nDie3","number").setValue(aRoll.aDice[3].result+aRoll.aDice[4].result);
	oNewNode.createChild("nDie4","number").setValue(aRoll.aDice[5].result);
	oNewNode.createChild("nDie5","number").setValue(aRoll.aDice[6].result);
	oNewNode.createChild("nDie6","number").setValue(aRoll.aDice[7].result);
	oNewNode.createChild("nEffectiveTempHigh","number").setValue(0);
	oNewNode.createChild("nEffectiveTempLow","number").setValue(0);
	oNewNode.createChild("nMonthDay","number").setValue(nMonthDay);
	oNewNode.createChild("nTempHigh","number").setValue(0);
	oNewNode.createChild("nTempLow","number").setValue(0);
	oNewNode.createChild("nWeatherDay","number").setValue(nWeatherDay);
	oNewNode.createChild("nWeatherMonth","number").setValue(nWeatherMonth);
	oNewNode.createChild("nWeatherYear","number").setValue(nWeatherYear);
	oNewNode.createChild("nWindGustingSpeed","number").setValue(0);
	oNewNode.createChild("nWindSpeed","number").setValue(0);
	oNewNode.createChild("sSpecialWeather","string").setValue(Interface.getString("sNoneStr"));
	oNewNode.createChild("sSunrise","string").setValue(" ");
	oNewNode.createChild("sSunset","string").setValue(" ");
	local oWindow = Interface.openWindow("wcWeather",oNewNode.getNodeName());
	if oWindow and oWindow.name then
		oWindow.name.setFocus();
	end
	fpSetRecordName(oNewNode);
	fpSetSunlight(oNewNode);
	fpCalcWeather(oNewNode)
	fpCalcWeather(oOldNode)
end

function fpGetClimateZone(oNode,nAdjustedDay,nYearLength)
	local nTilt = oNode.getChild("nTilt").getValue() or 22.5;
	local nLat = oNode.getChild("nLatitude").getValue() or 40;
	local nELat = math.abs(nLat+(nTilt-22.5)*math.cos(nAdjustedDay*2*math.pi/nYearLength));
	local sCZone = Interface.getString("sClimateTropicalStr");
	if nELat > 65 then
		sCZone = Interface.getString("sClimateArcticStr");
	elseif nELat > 50 then
		sCZone = Interface.getString("sClimateSubarcticStr");
	elseif nELat > 30 then
		sCZone = Interface.getString("sClimateTemperateStr");
	elseif nELat > 15 then
		sCZone = Interface.getString("sClimateSubtropicalStr");
	end
	return sCZone;
end

function fpGetEffectiveTemp(oNode)
	local nLow = oNode.getChild("nTempLow").getValue();
	local nHigh = oNode.getChild("nTempHigh").getValue();
	local nWindSpeed = oNode.getChild("nWindSpeed").getValue();
	local bFahrenheit = OptionsManager.isOption("TEMPMEASURE","fahrenheit");
	if not bFahrenheit then
		nLow = fpConvertCAndF(true,nLow)
		nHigh = fpConvertCAndF(true,nHigh)
	end
	if not OptionsManager.isOption("WINDMEASURE","mph") then
		nWindSpeed = fpConvertKPHAndMPH(true,nWindSpeed)
	end
	if OptionsManager.isOption("HUMIDITYON","on") then
		local sHumidity = oNode.getChild("sHumidity").getValue();
		if sHumidity == "" then
			sHumidity = Interface.getString("sHumidityMediumStr");
		end
		if sHumidity ~= Interface.getString("sHumidityDryStr") then
			local nWindVel = math.min(math.floor(nWindSpeed/10+0.5),5);
			if OptionsManager.isOption("WINDCHILLON","off") or
					nWindVel == 0 then
				nWindVel = 1;
			end
			if nLow >= 60 then
				nLow = nLow+WeatherTables.aHumidityWindTable[math.min(math.floor(nLow/10+0.5),10)][sHumidity][nWindVel];
			end
			if nHigh >= 60 then
				nHigh = nHigh+WeatherTables.aHumidityWindTable[math.min(math.floor(nHigh/10+0.5),10)][sHumidity][nWindVel];
			end
		end
	end
	if OptionsManager.isOption("WINDCHILLON","on") and
			nWindSpeed >= 3 then
		if nLow <= 50 then
			nLow = math.floor(35.74+0.6215*nLow-35.75*nWindSpeed^0.16+0.4275*nLow*nWindSpeed^0.16+0.5);
		end
		if nHigh <= 50 then
			nHigh = math.floor(35.74+0.6215*nHigh-35.75*nWindSpeed^0.16+0.4275*nHigh*nWindSpeed^0.16+0.5);
		end
	end
	if not bFahrenheit then
		nLow = fpConvertCAndF(false,nLow)
		nHigh = fpConvertCAndF(false,nHigh)
	end
	return nLow,nHigh;
end

function fpGetNode(oNode,bYesterdayNode)
	local sDay = "";
	if bYesterdayNode then
		if oNode.getChild("hsYesterdayLink") then
			sDay = oNode.getChild("hsYesterdayLink").getValue();
		end
	elseif oNode.getChild("hsTomorrowLink") then
		sDay = oNode.getChild("hsTomorrowLink").getValue();
	end
	if sDay == "" then
		return nil;
	else
		return DB.findNode(sDay);
	end
end

function fpGetRainfallWithDrought(oNode,sRainFall,bYSpecialWeatherD,bLongDrought)
	local sTmpRainfall = sRainFall;
	if bYSpecialWeatherD then
		sTmpRainfall = Interface.getString("sNoneStr");
	elseif sRainFall == Interface.getString("sPrecipitationAmountDownpourStr") then
		sTmpRainfall = Interface.getString("sPrecipitationAmountModerateStr");
	elseif sRainFall == Interface.getString("sPrecipitationAmountHeavyStr") then
		sTmpRainfall = Interface.getString("sPrecipitationAmountLightStr");
	elseif bLongDrought then
		if sRainFall == Interface.getString("sPrecipitationAmountModerateStr") or
				sRainFall == Interface.getString("sPrecipitationAmountLightStr") then
			sTmpRainfall = Interface.getString("sPrecipitationAmountTraceStr");
			oNode.getChild("hnDaysSinceDroughtTraceRainfall").setValue(0);
		end
	end
	return sTmpRainfall;
end

function fpGetRainfallWithSpecialWeather(oNode,sRainFall,sSpecialWeatherCode,nSpecialWeatherDuration,bYSpecialWeatherD,bLongDrought,bDesert)
	local sTmpRainFall = sRainFall;
--	local sTmpRainFall = Interface.getString("sNoneStr");
	if sSpecialWeatherCode == "c" then
		if nSpecialWeatherDuration == 3 then
			sTmpRainFall = Interface.getString("sPrecipitationAmountDownpourStr");
		elseif nSpecialWeatherDuration == 2 or
				nSpecialWeatherDuration == 1 then
			sTmpRainFall = Interface.getString("sNoneStr");
		end
	elseif sSpecialWeatherCode == "d" then
		sTmpRainFall = fpGetRainfallWithDrought(oNode,sRainFall,bYSpecialWeatherD,bLongDrought);
	elseif sSpecialWeatherCode == "g" and
			bDesert then
		sTmpRainFall = Interface.getString("sNoneStr");
	elseif sSpecialWeatherCode == "s" then
		sTmpRainFall = Interface.getString("sNoneStr");
	elseif sSpecialWeatherCode == "t" then
		sTmpRainFall = Interface.getString("sPrecipitationAmountHeavyStr");
	elseif sSpecialWeatherCode == "l" and
			(sRainFall == Interface.getString("sPrecipitationAmountModerateStr") or
			sRainFall == Interface.getString("sPrecipitationAmountLightStr") or
			sRainFall == Interface.getString("sPrecipitationAmountTraceStr")) then
		sTmpRainFall = Interface.getString("sNoneStr");
	end
	return sTmpRainFall;
end

function fpGetSeason(nMonth)
	local sSeason = "winter";
	if nMonth > 2 then
		if nMonth < 6 then
			sSeason = "spring";
		elseif nMonth < 9 then
			sSeason = "summer";
		elseif nMonth < 12 then
			sSeason = "autumn";
		end
	end
	return sSeason;
end

function fpGetSpinDirection(oNode)
	local sSpinDirection = Interface.getString("sSpinWestToEastStr");
	if oNode.getChild("sSpinDirection") then
		sSpinDirection = oNode.getChild("sSpinDirection").getValue();
	end
	if sSpinDirection == "" then
		sSpinDirection = Interface.getString("sSpinWestToEastStr");
	end
	if oNode.getChild("bIsPlane").getValue() == 1 then
		if sSpinDirection == Interface.getString("sSpinWestToEastStr") then
			sSpinDirection = Interface.getString("sSpinEastToWestStr");
		elseif sSpinDirection == Interface.getString("sSpinEastToWestStr") then
			sSpinDirection = Interface.getString("sSpinWestToEastStr");
		end
	end
	return sSpinDirection;
end

function fpGetSunriseSunset(nSunlighHours)
	local nHours,nMinutes = math.modf(nSunlighHours/2);
	nMinutes = math.floor(nMinutes*12+0.5)*5;
	local sMinutes = ":" .. nMinutes;
	if nMinutes == 0 then
		sMinutes = ":00";
	elseif nMinutes == 5 then
		sMinutes = ":05";
	end
	return("" .. nHours .. sMinutes);
end

function fpGetTempWithSpecialWeather(oNode,sSpecialWeatherCode,nSpecialWeatherDuration,nLow,nHigh,nMin,nMax,nDieTotal)
	local nTempChange = 0;
	local nTmpLow = nLow;
	local nTmpHigh = nHigh;
	local nTmpDieTotal = nDieTotal;
	local nDaysAtMin = 0;
	local nDaysAtMax = 0;
	if oYNode then
		nDaysAtMin = oYNode.getChild("hnDaysAtMin").getValue();
		nDaysAtMax = oYNode.getChild("hnDaysAtMax").getValue();
	end
	if sSpecialWeatherCode == "a" then
		nTmpLow = math.max(nLow-20,-50);
		nTmpHigh = math.max(nHigh-20,-30);
		if nTmpLow == -50 or
				nTmpHigh == -30 then
			oNode.getChild("hnSpecialWeatherDuration").setValue(math.min(nSpecialWeatherDuration,3));
		end
	elseif sSpecialWeatherCode == "z" then
		nTmpLow = math.min(nLow+20,95);
		nTmpHigh = math.min(nHigh+20,125);
		if nTmpLow == 95 or
				nTmpHigh == 125 then
			oNode.getChild("hnSpecialWeatherDuration").setValue(math.min(nSpecialWeatherDuration,3));
		end
	else
		if nTmpLow == nMin then
			if nTmpDieTotal < 8 then
				nTmpDieTotal = nTmpDieTotal+nDaysAtMin;
			end
			oNode.getChild("hnDaysAtMin").setValue(nDaysAtMin+1);
		elseif nTmpHigh == nMax then
			if nTmpDieTotal > 6 then
				nTmpDieTotal = nTmpDieTotal-nDaysAtMax;
			end
			oNode.getChild("hnDaysAtMax").setValue(nDaysAtMax+1);
		end
		nTempChange = 5*math.floor((nTmpDieTotal-7)/2+0.5);
		if sSpecialWeatherCode == "c" and
				(nSpecialWeatherDuration == 2 or
				nSpecialWeatherDuration == 1) then
			nTempChange = math.min(math.max(nTempChange,-5),5);
		elseif sSpecialWeatherCode == "t" then
			nTempChange = math.min(nTempChange,10);
		end
		if nTempChange < 0 then
			nTempChange = math.max(nMin-nTmpLow,nTempChange);
		elseif nTempChange > 0 then
			nTempChange = math.min(nMax-nTmpHigh,nTempChange);
		end
	end
	return nTmpLow,nTmpHigh,nTempChange;
end

function fpGetTerrain(oNode,sClimateZone)
	local sTTerrain = Interface.getString("sTerrainPlainsStr");
	if oNode.getChild("sTerrain") then
		sTTerrain = oNode.getChild("sTerrain").getValue();
	end
	local sRTerrain = sTTerrain;
	if sTTerrain == "" then
		sTTerrain = Interface.getString("sTerrainPlainsStr");
		sRTerrain = sTTerrain;
	elseif sTTerrain == Interface.getString("sTerrainForestMarshStr") then
		sTTerrain = Interface.getString("sTerrainForestStr");
		sRTerrain = "marsh";
	elseif sTTerrain == Interface.getString("sTerrainCoastMarshStr") then
		sTTerrain = Interface.getString("sTerrainCoastStr");
		sRTerrain = "marsh";
		if sClimateZone == Interface.getString("sClimateArcticStr") then
			sRTerrain = sTTerrain;
		end
	elseif sTTerrain == Interface.getString("sTerrainPlainsMarshStr") then
		sTTerrain = Interface.getString("sTerrainPlainsStr");
		sRTerrain = "marsh";
		if sClimateZone == Interface.getString("sClimateArcticStr") then
			sRTerrain = sTTerrain;
		end
	end
	if sClimateZone == Interface.getString("sClimateArcticStr") then
		if sTTerrain == Interface.getString("sTerrainForestStr") then
			sTTerrain = Interface.getString("sTerrainPlainsStr");
		end
		if sRTerrain == Interface.getString("sTerrainForestStr") then
			sRTerrain = Interface.getString("sTerrainPlainsStr");
		end
	end
	return sTTerrain,sRTerrain
end

function fpGetWindDirectionForClimate(nDie4,sClimateZone,nMonth,sOriginalSpinDirection,bNorthHemisphere)
	local sWindDirection;
	if sClimateZone == Interface.getString("sClimateArcticStr") then
		sWindDirection = fpGetWindDirectionForClimateArctic(nDie4,sOriginalSpinDirection);
	elseif sClimateZone == Interface.getString("sClimateSubarcticStr") then
		sWindDirection = fpGetWindDirectionForClimateSubArctic(nDie4,nMonth,sOriginalSpinDirection,bNorthHemisphere);
	elseif sClimateZone == Interface.getString("sClimateTemperateStr") then
		sWindDirection = fpGetWindDirectionForClimateTemperate(nDie4,sOriginalSpinDirection);
	else
		sWindDirection = fpGetWindDirectionForClimateTropic(nDie4,sOriginalSpinDirection,bNorthHemisphere);
	end
	return sWindDirection;
end

function fpGetWindDirectionForClimateArctic(nDie4,sOriginalSpinDirection)
	local sWindDirection = Interface.getString("sDirectionNorthWestStr");
	if sOriginalSpinDirection == Interface.getString("sSpinWestToEastStr") then
		sWindDirection = Interface.getString("sDirectionEastStr");
	elseif sOriginalSpinDirection == Interface.getString("sSpinEastToWestStr") then
		sWindDirection = Interface.getString("sDirectionWestStr");
	elseif nDie4 == 1 then
		sWindDirection = Interface.getString("sDirectionNorthStr");
	elseif nDie4 == 2 then
		sWindDirection = Interface.getString("sDirectionNorthEastStr");
	elseif nDie4 == 3 then
		sWindDirection = Interface.getString("sDirectionEastStr");
	elseif nDie4 == 4 then
		sWindDirection = Interface.getString("sDirectionSouthEastStr");
	elseif nDie4 == 5 then
		sWindDirection = Interface.getString("sDirectionSouthStr");
	elseif nDie4 == 6 then
		sWindDirection = Interface.getString("sDirectionSouthWestStr");
	elseif nDie4 == 7 then
		sWindDirection = Interface.getString("sDirectionWestStr");
	end
	return sWindDirection;
end

function fpGetWindDirectionForClimateSubArctic(nDie4,nMonth,sOriginalSpinDirection,bNorthHemisphere)
	local sWindDirection = Interface.getString("sDirectionSouthStr");
	if sOriginalSpinDirection == Interface.getString("sNoneStr") then
		if bNorthHemisphere then
			sWindDirection = Interface.getString("sDirectionNorthStr");
		else
			sWindDirection = Interface.getString("sDirectionSouthStr");
		end
	elseif nMonth > 4 and
			nMonth < 11 then
		if sOriginalSpinDirection == Interface.getString("sSpinWestToEastStr") then
			if nDie4 < 5 then
				sWindDirection = Interface.getString("sDirectionWestStr");
			elseif bNorthHemisphere then
				sWindDirection = Interface.getString("sDirectionSouthWestStr");
			else
				sWindDirection = Interface.getString("sDirectionNorthWestStr");
			end
		elseif nDie4 < 5 then
			sWindDirection = Interface.getString("sDirectionEastStr");
		elseif bNorthHemisphere then
			sWindDirection = Interface.getString("sDirectionSouthEastStr");
		else
			sWindDirection = Interface.getString("sDirectionNorthEastStr");
		end
	elseif sOriginalSpinDirection == Interface.getString("sSpinWestToEastStr") then
		if bNorthHemisphere then
			if nDie4 < 5 then
				sWindDirection = Interface.getString("sDirectionNorthWestStr");
			else
				sWindDirection = Interface.getString("sDirectionNorthStr");
			end
		elseif nDie4 < 5 then
			sWindDirection = Interface.getString("sDirectionSouthWestStr");
		else
			sWindDirection = Interface.getString("sDirectionSouthStr");
		end
	elseif bNorthHemisphere then
		if nDie4 < 5 then
			sWindDirection = Interface.getString("sDirectionNorthEastStr");
		else
			sWindDirection = Interface.getString("sDirectionNorthStr");
		end
	elseif nDie4 < 5 then
		sWindDirection = Interface.getString("sDirectionSouthEastStr");
	end
	return sWindDirection;
end

function fpGetWindDirectionForClimateTemperate(nDie4,sOriginalSpinDirection)
	local sWindDirection = Interface.getString("sDirectionEastStr");
	if nDie4 == 1 then
		sWindDirection = Interface.getString("sDirectionNorthStr");
	elseif nDie4 == 5 then
		sWindDirection = Interface.getString("sDirectionSouthStr");
	elseif sOriginalSpinDirection == Interface.getString("sNoneStr") then
		if nDie4 == 2 then
			sWindDirection = Interface.getString("sDirectionNorthEastStr");
		elseif nDie4 == 3 then
			sWindDirection = Interface.getString("sDirectionEastStr");
		elseif nDie4 == 4 then
			sWindDirection = Interface.getString("sDirectionSouthEastStr");
		elseif nDie4 == 6 then
			sWindDirection = Interface.getString("sDirectionSouthWestStr");
		elseif nDie4 == 7 then
			sWindDirection = Interface.getString("sDirectionWestStr");
		else
			sWindDirection = Interface.getString("sDirectionNorthWestStr");
		end
	elseif sOriginalSpinDirection == Interface.getString("sSpinWestToEastStr") then
		if nDie4 == 2 then
			sWindDirection = Interface.getString("sDirectionNorthStr");
		elseif nDie4 == 3 then
			sWindDirection = Interface.getString("sDirectionSouthStr");
		elseif nDie4 == 4 then
			sWindDirection = Interface.getString("sDirectionWestStr");
		elseif nDie4 == 6 then
			sWindDirection = Interface.getString("sDirectionSouthWestStr");
		elseif nDie4 == 7 then
			sWindDirection = Interface.getString("sDirectionWestStr");
		else
			sWindDirection = Interface.getString("sDirectionNorthWestStr");
		end
	elseif nDie4 == 2 then
		sWindDirection = Interface.getString("sDirectionNorthEastStr");
	elseif nDie4 == 3 then
		sWindDirection = Interface.getString("sDirectionEastStr");
	elseif nDie4 == 4 then
		sWindDirection = Interface.getString("sDirectionSouthEastStr");
	elseif nDie4 == 6 then
		sWindDirection = Interface.getString("sDirectionNorthStr");
	elseif nDie4 == 7 then
		sWindDirection = Interface.getString("sDirectionSouthStr");
	end
	return sWindDirection;
end

function fpGetWindDirectionForClimateTropic(nDie4,sOriginalSpinDirection,bNorthHemisphere)
	local sWindDirection = Interface.getString("sDirectionSouthWestStr");
	if sOriginalSpinDirection == Interface.getString("sNoneStr") then
		if bNorthHemisphere then
			sWindDirection = Interface.getString("sDirectionNorthStr");
		else
			sWindDirection = Interface.getString("sDirectionSouthStr");
		end
	elseif sOriginalSpinDirection == Interface.getString("sSpinWestToEastStr") then
		if nDie4 < 5 then
			sWindDirection = Interface.getString("sDirectionEastStr");
		elseif bNorthHemisphere then
			sWindDirection = Interface.getString("sDirectionNorthEastStr");
		else
			sWindDirection = Interface.getString("sDirectionSouthEastStr");
		end
	elseif nDie4 < 5 then
		sWindDirection = Interface.getString("sDirectionWestStr");
	elseif bNorthHemisphere then
		sWindDirection = Interface.getString("sDirectionNorthWestStr");
	end
	return sWindDirection;
end

function fpGetWindSpeed(nDieTotal)
	local nWindSpeedChange = 0;
	if nDieTotal == 2 then
		nWindSpeedChange = 23;
	elseif nDieTotal == 12 then
		nWindSpeedChange = -23;
	elseif nDieTotal == 7 then
		nWindSpeedChange = 0;
	elseif nDieTotal == 3 or
			nDieTotal == 5 then
		nWindSpeedChange = 8;
	elseif nDieTotal == 4 or
			nDieTotal == 6 then
		nWindSpeedChange = 5;
	elseif nDieTotal == 8 or
			nDieTotal == 10 then
		nWindSpeedChange = -5;
	else
		nWindSpeedChange = -8;
	end
	return nWindSpeedChange;
end

function fpGetYearLength(oNode)
	local nYearLength = oNode.getChild("nDaysInYear").getValue() or 365;
	local sYearMeasure = oNode.getChild("sYearMeasure").getValue();
	if sYearMeasure == Interface.getString("sTimeMinutesStr") then
		nYearLength = nYearLength/1440;
	elseif sYearMeasure == Interface.getString("sTimeHoursStr") then
		nYearLength = nYearLength/24;
	end
	return nYearLength;
end

function fpIsNodeShared(oNode,sUsername)
	if oNode then
		local aHolders = oNode.getHolders();
		if oNode.isPublic() then
			return true;
		elseif #aHolders > 0 then
			if not sUsername then
				return true;
			else
				for kKey,vValue in pairs(aHolders) do
					if vValue == sUsername then
						return true;
					end
				end
			end
		end
	end
	return false;
end

function fpRegisterWeatherOptions()
	OptionsManager.registerOption2("TEMPMEASURE",false,"sOptionHeaderWeather","sOptionLabelTemperature","option_entry_cycler",
			{labels = "sOptionValueFahrenheit",values = "fahrenheit",baselabel = "sOptionValueCelsius",baseval = "celsius",default = "celsius"});
	OptionsManager.registerOption2("WINDMEASURE",false,"sOptionHeaderWeather","sOptionLabelWindSpeed","option_entry_cycler",
			{labels = "sOptionValueMPH",values = "mph",baselabel = "sOptionValueKPH",baseval = "kph",default = "kph"});
	OptionsManager.registerOption2("HUMIDITYON",false,"sOptionHeaderWeather","sOptionLabelUseHumidity","option_entry_cycler",
			{labels = "option_val_on",values = "on",baselabel = "option_val_off",baseval = "off",default = "on"});
	OptionsManager.registerOption2("WINDCHILLON",false,"sOptionHeaderWeather","sOptionLabelUseWindChill","option_entry_cycler",
			{labels = "option_val_on",values = "on",baselabel = "option_val_off",baseval = "off",default = "on"});
	OptionsManager.registerCallback("TEMPMEASURE",fpChangeTempMeasure);
	OptionsManager.registerCallback("WINDMEASURE",fpChangeWindSpeedMeasure);
end

function fpSetExtremeRainCode(oNode,nDie6,sClimateZone,nTemp)
	local nDie5 = oNode.getChild("nDie5").getValue();
	if sClimateZone == Interface.getString("sClimateTropicalStr") or
			sClimateZone == Interface.getString("sClimateSubtropicalStr") then
		if nTemp > 49 then
			oNode.getChild("hsSpecialWeatherCode").setValue("l");
		elseif nTemp > 35 then
			if nDie5 == 6 then
				oNode.getChild("hsSpecialWeatherCode").setValue("h");
				oNode.getChild("hnSpecialWeatherDuration").setValue(nDie6+4);
			else
				oNode.getChild("hsSpecialWeatherCode").setValue("l");
			end
		elseif nTemp > 29 then
			oNode.getChild("hsSpecialWeatherCode").setValue("h");
			oNode.getChild("hnSpecialWeatherDuration").setValue(nDie6+4);
		end
	elseif nTemp > 74 then
		oNode.getChild("hsSpecialWeatherCode").setValue("l");
	elseif nTemp > 49 then
		if nDie5 == 6 then
			oNode.getChild("hsSpecialWeatherCode").setValue("h");
			oNode.getChild("hnSpecialWeatherDuration").setValue(nDie6+4);
		else
			oNode.getChild("hsSpecialWeatherCode").setValue("l");
		end
	elseif nTemp > 35 then
		if nDie5 > 4 then
			oNode.getChild("hsSpecialWeatherCode").setValue("h");
			oNode.getChild("hnSpecialWeatherDuration").setValue(nDie6+4);
		else
			oNode.getChild("hsSpecialWeatherCode").setValue("l");
		end
	elseif nTemp > 29 then
		if nDie5 == 6 then
			oNode.getChild("hsSpecialWeatherCode").setValue("i");
		elseif nDie5 == 5 then
			oNode.getChild("hsSpecialWeatherCode").setValue("h");
			oNode.getChild("hnSpecialWeatherDuration").setValue(nDie6+4);
		else
			oNode.getChild("hsSpecialWeatherCode").setValue("q");
		end
	elseif nTemp > 9 then
		if nDie5 == 6 then
			if nTemp > 24 then
				oNode.getChild("hsSpecialWeatherCode").setValue("i");
			end
		else
			oNode.getChild("hsSpecialWeatherCode").setValue("q");
		end
	elseif nTemp > -1 then
		if nDie5 < 6 then
			oNode.getChild("hsSpecialWeatherCode").setValue("q");
		end
	elseif nDie5 < 4 then
		oNode.getChild("hsSpecialWeatherCode").setValue("q");
	end
end

function fpSetHumidity(oNode,sClimateZone,sRainTerrain,sSeason)
	local sHumidity;
	if sClimateZone == Interface.getString("sClimateArcticStr") or
			sRainTerrain == Interface.getString("sTerrainDesertStr") then
		sHumidity = Interface.getString("sHumidityLowStr");
	else
		sHumidity = WeatherTables.aHumidityTable[sClimateZone][sRainTerrain][sSeason];
	end
	local sTRainFall = Interface.getString("sPrecipitationAmountTraceStr");
	local oTNode = fpGetNode(oNode,false);
	if oTNode and
			oTNode.getChild("sRainAmount") then
		sTRainFall = oTNode.getChild("sRainAmount").getValue();
	end
	if sTRainFall == "" or
			sTRainFall == Interface.getString("sNoneStr") then
		if sHumidity == Interface.getString("sHumiditySaturatedStr") then
			sHumidity = Interface.getString("sHumidityHeavyStr");
		elseif sHumidity == Interface.getString("sHumidityHeavyStr") then
			sHumidity = Interface.getString("sHumidityMediumStr");
		elseif sHumidity == Interface.getString("sHumidityMediumStr") then
			sHumidity = Interface.getString("sHumidityLowStr");
		else
			sHumidity = Interface.getString("sHumidityDryStr");
		end
	elseif sTRainFall == Interface.getString("sPrecipitationAmountModerateStr") or
			sTRainFall == Interface.getString("sPrecipitationAmountHeavyStr") or
			sTRainFall == Interface.getString("sPrecipitationAmountDownpourStr") then
		if sHumidity == Interface.getString("sHumidityDryStr") then
			sHumidity = Interface.getString("sHumidityLowStr");
		elseif sHumidity == Interface.getString("sHumidityLowStr") then
			sHumidity = Interface.getString("sHumidityMediumStr");
		elseif sHumidity == Interface.getString("sHumidityMediumStr") then
			sHumidity = Interface.getString("sHumidityHeavyStr");
		else
			sHumidity = Interface.getString("sHumiditySaturatedStr");
		end
	end
	oNode.createChild("sHumidity","string").setValue(sHumidity);
	return;
end

function fpSetRainFall(oNode,oYNode,nDieTotal,nDie1,nDie2,bDie4Odd,sClimateZone,sRainTerrain,sSeason,sSpecialWeatherCode,nSpecialWeatherDuration)
	local sRainFall = Interface.getString("sNoneStr");
	local bLongDrought = (oNode.getChild("hnDaysSinceDroughtTraceRainfall").getValue() > 5);
	local bYSpecialWeatherD = false;
	if oYNode then
		bYSpecialWeatherD = (oYNode.getChild("hsSpecialWeatherCode").getValue() == "d");
	end
	if nDieTotal == 7 then
		if oYNode then
			sRainFall = oYNode.getChild("sRainAmount").getValue();
		end
	elseif nDieTotal%2 ~= 0 then
		sRainFall = Interface.getString("sNoneStr");
	else
		local sCol = "c";
		local nYDie1 = 1;
		local nYDie2 = 6;
		if nDie1 > nDie2 then
			sCol = "a";
		elseif nDie1 == nDie2 then
			sCol = "b";
		end
		if oYNode then
			nYDie1 = oYNode.getChild("nDie1").getValue();
			nYDie2 = oYNode.getChild("nDie2").getValue();
		end
		sRainFall = WeatherTables.aRainTable[sClimateZone][sRainTerrain][sSeason][sCol];
		if nYDie1 == nYDie2 and
				nDie1 == nDie2 and
				nDieTotal ~= 2 and
				nDieTotal ~= 12 and
				(nYDie1+nYDie2) ~= 2 and
				(nYDie1+nYDie2) ~= 12 and
				not bDie4Odd then
			if sRainFall == Interface.getString("sNoneStr") then
				sRainFall = Interface.getString("sPrecipitationAmountTraceStr");
			elseif sRainFall == Interface.getString("sPrecipitationAmountTraceStr") then
				sRainFall = Interface.getString("sPrecipitationAmountLightStr");
			elseif sRainFall == Interface.getString("sPrecipitationAmountLightStr") then
				sRainFall = Interface.getString("sPrecipitationAmountModerateStr");
			elseif sRainFall == Interface.getString("sPrecipitationAmountModerateStr") then
				sRainFall = Interface.getString("sPrecipitationAmountHeavyStr");
			else
				sRainFall = Interface.getString("sPrecipitationAmountDownpourStr");
			end
		end
		sRainFall = fpAdjustRainfallForWetness(oNode,sRainFall)
	end
	oNode.createChild("sRainAmount","string").setValue(fpGetRainfallWithSpecialWeather(oNode,sRainFall,sSpecialWeatherCode,nSpecialWeatherDuration,bYSpecialWeatherD,bLongDrought,(sRainTerrain == Interface.getString("sTerrainDesertStr"))));
	return;
end

function fpSetRainHow(oNode,nWindSpeed)
	if not OptionsManager.isOption("WINDMEASURE","mph") then
		nWindSpeed = fpConvertKPHAndMPH(true,nWindSpeed)
	end
	if nWindSpeed > 18 then
		oNode.getChild("sRainHow").setValue(Interface.getString("sPrecipitationHowStormStr"));
	elseif nWindSpeed > 13 then
		oNode.getChild("sRainHow").setValue(Interface.getString("sPrecipitationHowHourStr"));
	elseif nWindSpeed > 5 then
		oNode.getChild("sRainHow").setValue(Interface.getString("sPrecipitationHowHalfDayStr"));
	elseif nWindSpeed > 3 then
		oNode.getChild("sRainHow").setValue(Interface.getString("sPrecipitationHowMostDayStr"));
	else
		oNode.getChild("sRainHow").setValue(Interface.getString("sPrecipitationHowDayStr"));
	end
end

function fpSetRecordName(oThisNode)
	local sLocationName = DB.getChild(oThisNode,"sWeatherLocationName").getValue() or Interface.getString("sUnknownStr");
	local bUseCal = (DB.getChild(oThisNode,"bUseCal").getValue() == 1) or false;
	local sDateStr = DB.getChild(oThisNode,"sCurrentDate").getValue() or Interface.getString("sNoneSetStr");
	local nMonth = DB.getChild(oThisNode,"nWeatherMonth").getValue() or 0;
	local nDay = DB.getChild(oThisNode,"nMonthDay").getValue() or 0;
	local nYear = DB.getChild(oThisNode,"nWeatherYear").getValue() or 0;
	local sNameStr = sLocationName;
	if sLocationName == "" then
		sNameStr = Interface.getString("sUnknownStr");
	end
	if not bUseCal then
		nDay = DB.getChild(oThisNode,"nWeatherDay").getValue() or 0;
		if nYear ~= 0 then
			sNameStr = sNameStr .. " (Year " .. nYear;
			if nDay ~= 0 then
				sNameStr = sNameStr .. " Day " .. nDay .. ")";
			else
				sNameStr = sNameStr .. ")";
			end
		elseif nDay ~= 0 then
			sNameStr = sNameStr .. " (Day " .. nDay .. ")";
		end
	elseif sDateStr ~= Interface.getString("sNoneSetStr") then
		if nYear ~= 0 then
			sNameStr = sNameStr .. ", " .. nYear .. ".";
		else
			sNameStr = sNameStr .. ", 00.";
		end
		if nMonth ~= 0 then
			sNameStr = sNameStr .. nMonth .. ".";
		else
			sNameStr = sNameStr .. "00.";
		end
		if nDay ~= 0 then
			sNameStr = sNameStr .. nDay;
		else
			sNameStr = sNameStr .. "00";
		end
	end
	DB.setValue(oThisNode,"name","string",sNameStr);
end

function fpSetSnowOrRain(oNode,nLow,nHigh)
	if not OptionsManager.isOption("TEMPMEASURE","fahrenheit") then
		nLow = fpConvertCAndF(true,nLow)
		nHigh = fpConvertCAndF(true,nHigh)
	end
	if nLow > 35 then
		oNode.getChild("sRainType").setValue(Interface.getString("sPrecipitationTypeRainStr"));
	elseif nHigh < 30 then
		oNode.getChild("sRainType").setValue(Interface.getString("sPrecipitationTypeSnowStr"));
	else
		oNode.getChild("sRainType").setValue(Interface.getString("sPrecipitationTypeSleetStr"));
	end
end

function fpSetSpecialWeatherCode(oNode,oYNode,nDieTotal,nDie6,sClimateZone,sRainTerrain,sSeason)
	local nDie3 = oNode.getChild("nDie3").getValue();
	local nDie5 = oNode.getChild("nDie5").getValue();
	local sCol = "e";
	local sSpecialWeatherCode = "n";
	local sYSpecialWeather = "n";
	if oYNode then
		sYSpecialWeather = oYNode.getChild("hsSpecialWeatherCode").getValue();
	end
	if (sYSpecialWeather == "a" or
			sYSpecialWeather == "c" or
			sYSpecialWeather == "d" or
			sYSpecialWeather == "z") and
			oNode.getChild("hnSpecialWeatherDuration").getValue() ~= 0 then
		sSpecialWeatherCode = sYSpecialWeather;
	elseif nDieTotal == 2 then
		if nDie3 < 5 then
			sCol = "a";
		elseif nDie3 < 8 then
			sCol = "b";
		end
	elseif nDieTotal == 12 then
		if nDie3 > 9 then
			sCol = "d";
		elseif nDie3 > 6 then
			sCol = "c";
		end
	end
	if sCol ~= "e" then
		sSpecialWeatherCode = WeatherTables.aSpecialWeatherTable[sClimateZone][sRainTerrain][sSeason][sCol];
		if sSpecialWeatherCode == "a" then
			oNode.getChild("hnSpecialWeatherDuration").setValue(nDie5+2);
		elseif sSpecialWeatherCode == "z" then
			oNode.getChild("hnSpecialWeatherDuration").setValue(nDie5+2);
		elseif sSpecialWeatherCode == "c" then
			if nLat <= 5 then
				sSpecialWeatherCode = "n";
			elseif oNode.getChild("hnDaysSinceCyclone").getValue() < 31 then
				sSpecialWeatherCode = "x"
			else
				oNode.getChild("hnSpecialWeatherDuration").setValue(3);
				oNode.getChild("hnDaysSinceCyclone").setValue(0);
			end
		elseif sSpecialWeatherCode == "d" then
			oNode.getChild("hnSpecialWeatherDuration").setValue(nDie5+nDie6+2);
			oNode.getChild("hnDaysSinceDroughtTraceRainfall").setValue(0);
		elseif sSpecialWeatherCode == "g" then
			oNode.getChild("hnSpecialWeatherDuration").setValue(nDie5+1);
		elseif sSpecialWeatherCode == "m" then
			oNode.getChild("hnSpecialWeatherDuration").setValue(nDie5);
		elseif sSpecialWeatherCode == "s" then
			oNode.getChild("hnSpecialWeatherDuration").setValue(nDie5+1);
		elseif sSpecialWeatherCode == "t" then
			oNode.getChild("hnSpecialWeatherDuration").setValue(math.ceil(nDie5/2));
		end
	end
	oNode.createChild("hsSpecialWeatherCode","string").setValue(sSpecialWeatherCode);
	return sSpecialWeatherCode,oNode.getChild("hnSpecialWeatherDuration","number").getValue();
end

function fpSetSpecialWeatherText(oNode,sRainTerrain,nWindSpeed,sSpecialWeatherCode)
	if not OptionsManager.isOption("WINDMEASURE","mph") then
		nWindSpeed = fpConvertKPHAndMPH(true,nWindSpeed)
	end
	if sSpecialWeatherCode == "a" then
		oNode.getChild("sSpecialWeather").setValue(Interface.getString("sSpecialWeather01Str"));
	elseif sSpecialWeatherCode == "c" then
		if nWindSpeed > 70 then
			oNode.getChild("sSpecialWeather").setValue(Interface.getString("sSpecialWeather03Str") .. " " .. Interface.getString("sSpecialWeather02Str"));
		elseif nWindSpeed > 40 then
			oNode.getChild("sSpecialWeather").setValue(Interface.getString("sSpecialWeather04Str") .. " " .. Interface.getString("sSpecialWeather02Str"));
		else
			oNode.getChild("sSpecialWeather").setValue(Interface.getString("sSpecialWeather05Str") .. " " .. Interface.getString("sSpecialWeather02Str"));
		end
		if sRainTerrain == Interface.getString("sTerrainCoastStr") then
			local oYNode = fpGetNode(oNode,true);
			if oYNode then
				oYNode.getChild("sSpecialWeather").setValue(Interface.getString("sSpecialWeather07Str") .. " " .. Interface.getString("sSpecialWeather06Str"));
				oYNode.getChild("hsSpecialWeatherCode").setValue("l");
			end
		end
	elseif sSpecialWeatherCode == "d" then
		oNode.getChild("sSpecialWeather").setValue(Interface.getString("sSpecialWeather08Str"));
	elseif sSpecialWeatherCode == "g" then
		if sRainTerrain == Interface.getString("sTerrainDesertStr") then
			oNode.getChild("sSpecialWeather").setValue(Interface.getString("sSpecialWeather09Str") .. " " .. Interface.getString("sSpecialWeather10Str") .. " " .. oNode.getChild("hnSpecialWeatherDuration").getValue() .. " " .. Interface.getString("sSpecialWeather11Str"));
		else
			oNode.getChild("sSpecialWeather").setValue(Interface.getString("sSpecialWeather12Str") .. " " .. Interface.getString("sSpecialWeather10Str") .. " " .. oNode.getChild("hnSpecialWeatherDuration").getValue() .. " " .. Interface.getString("sSpecialWeather11Str"));
		end
	elseif sSpecialWeatherCode == "h" then
		oNode.getChild("sSpecialWeather").setValue(Interface.getString("sSpecialWeather13Str") .. " " .. Interface.getString("sSpecialWeather10Str") .. " " .. oNode.getChild("hnSpecialWeatherDuration").getValue() .. " " .. Interface.getString("sSpecialWeather14Str"));
	elseif sSpecialWeatherCode == "i" then
		oNode.getChild("sSpecialWeather").setValue(Interface.getString("sSpecialWeather15Str") .. " " .. Interface.getString("sSpecialWeather06Str"));
	elseif sSpecialWeatherCode == "l" then
		oNode.getChild("sSpecialWeather").setValue(Interface.getString("sSpecialWeather07Str") .. " " .. Interface.getString("sSpecialWeather06Str"));
	elseif sSpecialWeatherCode == "m" then
		if sRainTerrain == Interface.getString("sTerrainCoastStr") then
			if oNode.getChild("hnSpecialWeatherDuration").getValue() < 5 then
				oNode.getChild("sSpecialWeather").setValue(Interface.getString("sPrecipitationAmountHeavyStr") .. " " .. Interface.getString("sSpecialWeather16Str") .. " " .. Interface.getString("sSunriseStr") .. " " .. Interface.getString("sSpecialWeather10Str") .. " " .. oNode.getChild("hnSpecialWeatherDuration").getValue() .. " " .. Interface.getString("sSpecialWeather11Str"));
			else
				oNode.getChild("sSpecialWeather").setValue(Interface.getString("sPrecipitationAmountHeavyStr") .. " " .. Interface.getString("sSpecialWeather16Str") .. " " .. Interface.getString("sSunsetStr") .. " " .. Interface.getString("sSpecialWeather10Str") .. " " .. oNode.getChild("hnSpecialWeatherDuration").getValue() .. " " .. Interface.getString("sSpecialWeather11Str"));
			end
		elseif oNode.getChild("hnSpecialWeatherDuration").getValue() < 5 then
			oNode.getChild("sSpecialWeather").setValue(Interface.getString("sSpecialWeather04Str") .. " " .. Interface.getString("sSpecialWeather16Str") .. " " .. Interface.getString("sSunriseStr") .. " " .. Interface.getString("sSpecialWeather10Str") .. " " .. oNode.getChild("hnSpecialWeatherDuration").getValue() .. " " .. Interface.getString("sSpecialWeather11Str"));
		else
			oNode.getChild("sSpecialWeather").setValue(Interface.getString("sSpecialWeather04Str") .. " " .. Interface.getString("sSpecialWeather16Str") .. " " .. Interface.getString("sSunsetStr") .. " " .. Interface.getString("sSpecialWeather10Str") .. " " .. oNode.getChild("hnSpecialWeatherDuration").getValue() .. " " .. Interface.getString("sSpecialWeather11Str"));
		end
	elseif sSpecialWeatherCode == "q" then
		local sRainAmount = oNode.getChild("sRainAmount").getValue();
		local nDie5 = oNode.getChild("nDie5").getValue();
		local nDuration = 0;
		if sRainAmount == Interface.getString("sPrecipitationAmountHeavyStr") or
				sRainAmount == Interface.getString("sPrecipitationAmountDownpourStr") then
			nDuration = nDie5+6;
		else
			nDuration = nDie5+4;
		end
		oNode.getChild("hnSpecialWeatherDuration").setValue(nDuration);
		oNode.getChild("sSpecialWeather").setValue(Interface.getString("sSpecialWeather17Str") .. " " .. Interface.getString("sSpecialWeather10Str") .. " " .. nDuration .. " " .. Interface.getString("sSpecialWeather11Str"));
	elseif sSpecialWeatherCode == "s" then
		oNode.getChild("sSpecialWeather").setValue(Interface.getString("sSpecialWeather18Str") .. " " .. Interface.getString("sSpecialWeather10Str") .. " " .. oNode.getChild("hnSpecialWeatherDuration").getValue() .. " " .. Interface.getString("sSpecialWeather11Str"));
	elseif sSpecialWeatherCode == "t" then
		oNode.getChild("sSpecialWeather").setValue(Interface.getString("sSpecialWeather20Str"));
	elseif sSpecialWeatherCode == "z" then
		oNode.getChild("sSpecialWeather").setValue(Interface.getString("sSpecialWeather19Str"));
	else
		oNode.getChild("sSpecialWeather").setValue(Interface.getString("sNoneStr"));
	end
end

function fpSetSunlight(oNode)
	local nLat = math.tan(math.rad(oNode.getChild("nLatitude").getValue()));
	if oNode.getChild("sLatHemisphere") and
			oNode.getChild("sLatHemisphere").getValue() == Interface.getString("sDirectionSouthStr") then
		nLat = 0-nLat;
	end
	local nTilt = oNode.getChild("nTilt").getValue() or 22.5;
	local nDay = oNode.getChild("nWeatherDay").getValue() or 1;
	local nYearLength = fpGetYearLength(oNode);
	local nSumSol = oNode.getChild("nSumSolDay").getValue() or SUMSOLDAY;
	local nHoursInDay = oNode.getChild("nHoursInDay").getValue() or 24;
	local sDayMeasure = oNode.getChild("sDayMeasure").getValue();
	if sDayMeasure == Interface.getString("sTimeMinutesStr") then
		nHoursInDay = nHoursInDay/60;
	elseif sDayMeasure == Interface.getString("sTimeDaysStr") then
		nHoursInDay = nHoursInDay*24;
	end
	local nHoursOfSunlight = math.floor((math.acos(1-math.min(math.max(1-nLat*math.tan(math.rad(nTilt*math.cos(fpAdjustedDayOfYear(nDay,nYearLength,nSumSol)*2*math.pi/nYearLength))),0),2))*nHoursInDay/math.pi)*6+0.5)/6;
	if nHoursOfSunlight >= nHoursInDay then
		oNode.getChild("sSunrise").setValue("Midnight Sun");
		oNode.getChild("sSunset").setValue("Midnight Sun");
	elseif nHoursOfSunlight <= 0 then
		oNode.getChild("sSunrise").setValue("Polar Night");
		oNode.getChild("sSunset").setValue("Polar Night");
	else
		oNode.getChild("sSunrise").setValue(fpGetSunriseSunset(nHoursInDay-nHoursOfSunlight));
		oNode.getChild("sSunset").setValue(fpGetSunriseSunset(nHoursOfSunlight));
	end
end

function fpSetTemp(oNode,oYNode,nDieTotal,nDie6,sClimateZone,nMonth,sTempTerrain,sSpecialWeatherCode,nSpecialWeatherDuration)
	local nLow = WeatherTables.aTempTable[sClimateZone][sTempTerrain][nMonth]["low"];
	local nHigh = WeatherTables.aTempTable[sClimateZone][sTempTerrain][nMonth]["high"];
	local nMin = WeatherTables.aTempTable[sClimateZone][sTempTerrain][nMonth]["min"];
	local nMax = WeatherTables.aTempTable[sClimateZone][sTempTerrain][nMonth]["max"];
	local bFahrenheit = OptionsManager.isOption("TEMPMEASURE","fahrenheit");
	if oYNode then
		nLow = oYNode.getChild("nTempLow").getValue();
		nHigh = oYNode.getChild("nTempHigh").getValue();
		if not bFahrenheit then
			nLow = fpConvertCAndF(true,nLow)
			nHigh = fpConvertCAndF(true,nHigh)
		end
	end
	nLow,nHigh = fpAdjustTempForAltitude(oNode,fpGetTempWithSpecialWeather(oNode,sSpecialWeatherCode,nSpecialWeatherDuration,nLow,nHigh,nMin,nMax,nDieTotal));
	if sSpecialWeatherCode == "x" then
		fpSetExtremeRainCode(oNode,nDie6,sClimateZone,(nLow+nHigh)/2);
	end
	nLow,nHigh = fpAdjustTempForHotness(oNode,nLow,nHigh)
	if not bFahrenheit then
		nLow = fpConvertCAndF(false,nLow);
		nHigh = fpConvertCAndF(false,nHigh);
	end
	oNode.getChild("nTempLow").setValue(nLow);
	oNode.getChild("nTempHigh").setValue(nHigh);
	return;
end

function fpSetWindDirection(oNode,nDieTotal,nDie4,sClimateZone,nMonth,sOriginalSpinDirection,bNorthHemisphere,sSpecialWeatherCode)
	local sWindDirection = fpGetWindDirectionForClimate(nDie4,sClimateZone,nMonth,sOriginalSpinDirection,bNorthHemisphere);
	if nDieTotal == 3 or
			nDieTotal == 4 then
		sWindDirection = Interface.getString("sDirectionSouthStr");
		if bNorthHemisphere then
			sWindDirection = Interface.getString("sDirectionNorthStr");
		end
	elseif nDieTotal == 10 or
			nDieTotal == 11 then
		sWindDirection = Interface.getString("sDirectionNorthStr");
		if bNorthHemisphere then
			sWindDirection = Interface.getString("sDirectionSouthStr");
		end
	elseif sSpecialWeatherCode == "n" and
			nDie4 < 5 then
		if nDieTotal == 2 then
			sWindDirection = Interface.getString("sDirectionSouthStr");
			if bNorthHemisphere then
				sWindDirection = Interface.getString("sDirectionNorthStr");
			end
		elseif nDieTotal == 12 then
			sWindDirection = Interface.getString("sDirectionNorthStr");
			if bNorthHemisphere then
				sWindDirection = Interface.getString("sDirectionSouthStr");
			end
		end
	end
	oNode.createChild("sWindDirection","string").setValue(sWindDirection);
	return;
end

function fpSetWindSpeeed(oNode,oYNode,nDieTotal,nDie6,sClimateZone,sSpecialWeatherCode,nSpecialWeatherDuration)
	local bMPH = OptionsManager.isOption("WINDMEASURE","mph");
	local nWindSpeed = 0;
	if oYNode then
		nWindSpeed = oYNode.getChild("nWindSpeed").getValue();
	end
	if not bMPH then
		nWindSpeed = fpConvertKPHAndMPH(true,nWindSpeed)
	end
	local nMaxWind = 23;
	if sClimateZone == Interface.getString("sClimateArcticStr") or
			sClimateZone == Interface.getString("sClimateTropicalStr") then
		nMaxWind = 10;
	elseif sClimateZone == Interface.getString("sClimateSubarcticStr") then
		nMaxWind = 15;
	end
	nWindSpeed = math.max(math.min(nWindSpeed+fpGetWindSpeed(nDieTotal),nMaxWind),0);
	if sSpecialWeatherCode == "c" then
		if nSpecialWeatherDuration == 3 then
			nWindSpeed = nDie6*10+30;
		elseif nSpecialWeatherDuration == 2 or
				nSpecialWeatherDuration == 1 then
			nWindSpeed = 5;
		end
	elseif sSpecialWeatherCode == "g" or
			sSpecialWeatherCode == "s" or
			sSpecialWeatherCode == "t" then
		nWindSpeed = nDie6*3+20;
	end
	if not bMPH then
		nWindSpeed = fpConvertKPHAndMPH(true,nWindSpeed)
	end
	oNode.getChild("nWindSpeed").setValue(nWindSpeed);
	return nWindSpeed;
end
