--
-- © Copyright Peregrine I.T. Pty Ltd 2004-2018 except where explicitly stated otherwise.
-- Fantasy Grounds is Copyright © 2004-2018 SmiteWorks USA LLC.
-- Copyright to other material within this file may be held by other Individuals and/or Entities.
-- Nothing in or from this LUA file in printed, electronic and/or any other form may be used, copied,
--	transmitted or otherwise manipulated in ANY way without the explicit written consent of
--	Peregrine I.T. Pty Ltd or, where applicable, any and all other Copyright holders.
-- Please see the accompanying License for full details.
-- All rights reserved.
--
-- Called By: wcWeather in xWeatherWindowClasses.xml
--
function onInit()
	OptionsManager.registerCallback("HUMIDITYON",fpStateChanged);
	OptionsManager.registerCallback("WINDCHILLON",fpStateChanged);
	getDatabaseNode().createChild("soundlist");
	fpStateChanged();
end

function onClose()
	OptionsManager.unregisterCallback("HUMIDITYON",fpStateChanged);
	OptionsManager.unregisterCallback("WINDCHILLON",fpStateChanged);
end

function onLockChanged()
	fpStateChanged();
end

function fpStateChanged()
	local oNode = getDatabaseNode();
	local nMin,nMax = WeatherManager.fpGetEffectiveTemp(oNode);
	local bLocked = MiscFunctions.fpGetReadOnlyState(oNode);
	local bFooter = not bLocked;
	swMain.subwindow.nEffectiveTempLow.setValue(nMin);
	swMain.subwindow.nEffectiveTempHigh.setValue(nMax);
	if swMain.subwindow then
		swMain.subwindow.fpUpdate();
	end
	if swDetails.subwindow then
		swDetails.subwindow.fpUpdate();
	end
	if swSounds.subwindow then
		swSounds.subwindow.fpUpdate();
	end
	if swNotes.subwindow then
		swNotes.subwindow.fpUpdate();
	end
	bcGenerateTomorrowsWeather.setTooltipText(Interface.getString("sGenerateNextDaysWeatherStr"))
	if oNode.getChild("hsTomorrowLink") and
			oNode.getChild("hsTomorrowLink").getValue() ~= "" then
		local oTomorrowsNode = DB.findNode(oNode.getChild("hsTomorrowLink").getValue());
		if (User.isHost() or
				WeatherManager.fpIsNodeShared(oTomorrowsNode,User.getUsername())) and
				oTomorrowsNode ~= nil then
			bFooter = true;
			bcGenerateTomorrowsWeather.setTooltipText(Interface.getString("sOpenNextDaysWeatherStr"))
		end
	end
	bcGenerateTodaysWeather.setVisible(not bLocked);
	bcGenerateTomorrowsWeather.setVisible(bFooter);
	vFrame.setAnchor("bottom","","bottom","",-30);
	swMain.setAnchor("bottom","","bottom","",-30);
	swDetails.setAnchor("bottom","","bottom","",-30);
	swNotes.setAnchor("bottom","","bottom","",-30);
	if bFooter then
		vFrame.setAnchor("bottom","","bottom","",-65);
		swMain.setAnchor("bottom","","bottom","",-65);
		swDetails.setAnchor("bottom","","bottom","",-65);
		swNotes.setAnchor("bottom","","bottom","",-65);
	end
end
